@@ -167,11 +167,26 @@ class ServerDelete(task_manager.Task):
         return client.nova_client.servers.delete(**self.args)
 
 
+class ServerUpdate(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.servers.update(**self.args)
+
+
 class ServerRebuild(task_manager.Task):
     def main(self, client):
         return client.nova_client.servers.rebuild(**self.args)
 
 
+class ServerSetMetadata(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.servers.set_meta(**self.args)
+
+
+class ServerDeleteMetadata(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.servers.delete_meta(**self.args)
+
+
 class ServerGroupList(task_manager.Task):
     def main(self, client):
         return client.nova_client.server_groups.list(**self.args)
@@ -546,7 +561,7 @@ class ObjectCreate(task_manager.Task):
 
 class ObjectUpdate(task_manager.Task):
     def main(self, client):
-        client.swift_client.post_object(**self.args)
+        return client.swift_client.post_object(**self.args)
 
 
 class ObjectList(task_manager.Task):
@@ -877,3 +892,74 @@ class RecordSetUpdate(task_manager.Task):
 class RecordSetDelete(task_manager.Task):
     def main(self, client):
         return client.designate_client.recordsets.delete(**self.args)
+
+
+class NovaQuotasSet(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.quotas.update(**self.args)
+
+
+class NovaQuotasGet(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.quotas.get(**self.args)
+
+
+class NovaQuotasDelete(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.quotas.delete(**self.args)
+
+
+class CinderQuotasSet(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.quotas.update(**self.args)
+
+
+class CinderQuotasGet(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.quotas.get(**self.args)
+
+
+class CinderQuotasDelete(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.quotas.delete(**self.args)
+
+
+class NeutronQuotasSet(task_manager.Task):
+    def main(self, client):
+        return client.neutron_client.update_quota(**self.args)
+
+
+class NeutronQuotasGet(task_manager.Task):
+    def main(self, client):
+        return client.neutron_client.show_quota(**self.args)['quota']
+
+
+class NeutronQuotasDelete(task_manager.Task):
+    def main(self, client):
+        return client.neutron_client.delete_quota(**self.args)
+
+
+class ClusterTemplateList(task_manager.Task):
+    def main(self, client):
+        return client.magnum_client.baymodels.list(**self.args)
+
+
+class ClusterTemplateCreate(task_manager.Task):
+    def main(self, client):
+        return client.magnum_client.baymodels.create(**self.args)
+
+
+class ClusterTemplateDelete(task_manager.Task):
+    def main(self, client):
+        return client.magnum_client.baymodels.delete(self.args['id'])
+
+
+class ClusterTemplateUpdate(task_manager.Task):
+    def main(self, client):
+        return client.magnum_client.baymodels.update(
+            self.args['id'], self.args['patch'])
+
+
+class MagnumServicesList(task_manager.Task):
+    def main(self, client):
+        return client.magnum_client.mservices.list(detail=False)
