@@ -97,30 +97,9 @@ _TASK_SCHEMA = dict(
 
 class TestMemoryCache(base.TestCase):
 
-    CLOUD_CONFIG = {
-        'cache':
-        {
-            'max_age': 90,
-            'class': 'dogpile.cache.memory',
-            'expiration': {
-                'server': 1,
-            },
-        },
-        'clouds':
-        {
-            '_test_cloud_':
-            {
-                'auth':
-                {
-                    'auth_url': 'http://198.51.100.1:35357/v2.0',
-                    'username': '_test_user_',
-                    'password': '_test_pass_',
-                    'project_name': '_test_project_',
-                },
-                'region_name': '_test_region_',
-            },
-        },
-    }
+    def setUp(self):
+        super(TestMemoryCache, self).setUp(
+            cloud_config_fixture='clouds_cache.yaml')
 
     def test_openstack_cloud(self):
         self.assertIsInstance(self.cloud, shade.OpenStackCloud)
@@ -407,7 +386,7 @@ class TestMemoryCache(base.TestCase):
         fake_image = fakes.FakeImage('42', '42 name', 'success')
         glance_mock.images.create.return_value = fake_image
         glance_mock.images.list.return_value = [fake_image]
-        self._call_create_image('42 name', min_disk=0, min_ram=0)
+        self._call_create_image('42 name', min_disk='0', min_ram=0)
         args = {'name': '42 name',
                 'container_format': 'bare', 'disk_format': 'qcow2',
                 'owner_specified.shade.md5': mock.ANY,
@@ -421,6 +400,106 @@ class TestMemoryCache(base.TestCase):
         fake_image_dict = meta.obj_to_dict(fake_image)
         self.assertEqual([fake_image_dict], self.cloud.list_images())
 
+    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, 'glance_client')
+    def test_create_image_put_bad_int(self, glance_mock, mock_api_version):
+        mock_api_version.return_value = '2'
+        self.cloud.image_api_use_tasks = False
+
+        glance_mock.images.list.return_value = []
+        self.assertEqual([], self.cloud.list_images())
+
+        fake_image = fakes.FakeImage('42', '42 name', 'success')
+        glance_mock.images.create.return_value = fake_image
+        glance_mock.images.list.return_value = [fake_image]
+        self.assertRaises(
+            exc.OpenStackCloudException,
+            self._call_create_image, '42 name', min_disk='fish', min_ram=0)
+
+    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, 'glance_client')
+    def test_create_image_put_user_int(self, glance_mock, mock_api_version):
+        mock_api_version.return_value = '2'
+        self.cloud.image_api_use_tasks = False
+
+        glance_mock.images.list.return_value = []
+        self.assertEqual([], self.cloud.list_images())
+
+        fake_image = fakes.FakeImage('42', '42 name', 'success')
+        glance_mock.images.create.return_value = fake_image
+        glance_mock.images.list.return_value = [fake_image]
+        self._call_create_image(
+            '42 name', min_disk='0', min_ram=0, int_v=12345)
+        args = {'name': '42 name',
+                'container_format': 'bare', 'disk_format': u'qcow2',
+                'owner_specified.shade.md5': mock.ANY,
+                'owner_specified.shade.sha256': mock.ANY,
+                'owner_specified.shade.object': 'images/42 name',
+                'int_v': '12345',
+                'visibility': 'private',
+                'min_disk': 0, 'min_ram': 0}
+        glance_mock.images.create.assert_called_with(**args)
+        glance_mock.images.upload.assert_called_with(
+            image_data=mock.ANY, image_id=fake_image.id)
+        fake_image_dict = meta.obj_to_dict(fake_image)
+        self.assertEqual([fake_image_dict], self.cloud.list_images())
+
+    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, 'glance_client')
+    def test_create_image_put_meta_int(self, glance_mock, mock_api_version):
+        mock_api_version.return_value = '2'
+        self.cloud.image_api_use_tasks = False
+
+        glance_mock.images.list.return_value = []
+        self.assertEqual([], self.cloud.list_images())
+
+        fake_image = fakes.FakeImage('42', '42 name', 'success')
+        glance_mock.images.create.return_value = fake_image
+        glance_mock.images.list.return_value = [fake_image]
+        self._call_create_image(
+            '42 name', min_disk='0', min_ram=0, meta={'int_v': 12345})
+        args = {'name': '42 name',
+                'container_format': 'bare', 'disk_format': u'qcow2',
+                'owner_specified.shade.md5': mock.ANY,
+                'owner_specified.shade.sha256': mock.ANY,
+                'owner_specified.shade.object': 'images/42 name',
+                'int_v': 12345,
+                'visibility': 'private',
+                'min_disk': 0, 'min_ram': 0}
+        glance_mock.images.create.assert_called_with(**args)
+        glance_mock.images.upload.assert_called_with(
+            image_data=mock.ANY, image_id=fake_image.id)
+        fake_image_dict = meta.obj_to_dict(fake_image)
+        self.assertEqual([fake_image_dict], self.cloud.list_images())
+
+    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, 'glance_client')
+    def test_create_image_put_user_prop(self, glance_mock, mock_api_version):
+        mock_api_version.return_value = '2'
+        self.cloud.image_api_use_tasks = False
+
+        glance_mock.images.list.return_value = []
+        self.assertEqual([], self.cloud.list_images())
+
+        fake_image = fakes.FakeImage('42', '42 name', 'success')
+        glance_mock.images.create.return_value = fake_image
+        glance_mock.images.list.return_value = [fake_image]
+        self._call_create_image(
+            '42 name', min_disk='0', min_ram=0, properties={'int_v': 12345})
+        args = {'name': '42 name',
+                'container_format': 'bare', 'disk_format': u'qcow2',
+                'owner_specified.shade.md5': mock.ANY,
+                'owner_specified.shade.sha256': mock.ANY,
+                'owner_specified.shade.object': 'images/42 name',
+                'int_v': '12345',
+                'visibility': 'private',
+                'min_disk': 0, 'min_ram': 0}
+        glance_mock.images.create.assert_called_with(**args)
+        glance_mock.images.upload.assert_called_with(
+            image_data=mock.ANY, image_id=fake_image.id)
+        fake_image_dict = meta.obj_to_dict(fake_image)
+        self.assertEqual([fake_image_dict], self.cloud.list_images())
+
     @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
     @mock.patch.object(shade.OpenStackCloud, '_get_file_hashes')
     @mock.patch.object(shade.OpenStackCloud, 'glance_client')
@@ -522,37 +601,10 @@ class TestMemoryCache(base.TestCase):
 
 
 class TestBogusAuth(base.TestCase):
-    CLOUD_CONFIG = {
-        'clouds':
-        {
-            '_test_cloud_':
-            {
-                'auth':
-                {
-                    'auth_url': 'http://198.51.100.1:35357/v2.0',
-                    'username': '_test_user_',
-                    'password': '_test_pass_',
-                    'project_name': '_test_project_',
-                },
-                'region_name': '_test_region_',
-            },
-            '_bogus_test_':
-            {
-                'auth_type': 'bogus',
-                'auth':
-                {
-                    'auth_url': 'http://198.51.100.1:35357/v2.0',
-                    'username': '_test_user_',
-                    'password': '_test_pass_',
-                    'project_name': '_test_project_',
-                },
-                'region_name': '_test_region_',
-            },
-        },
-    }
 
     def setUp(self):
-        super(TestBogusAuth, self).setUp()
+        super(TestBogusAuth, self).setUp(
+            cloud_config_fixture='clouds_cache.yaml')
 
     def test_get_auth_bogus(self):
         with testtools.ExpectedException(exc.OpenStackCloudException):
