@@ -16,6 +16,8 @@
 import mock
 
 import shade
+from keystoneauth1.fixture import keystoneauth_betamax
+from keystoneauth1.fixture import serializer
 from shade.tests import fakes
 from shade.tests.unit import base
 
@@ -24,23 +26,46 @@ class TestFlavors(base.TestCase):
 
     def setUp(self):
         super(TestFlavors, self).setUp()
-        self.op_cloud = shade.operator_cloud()
 
-    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
-    def test_create_flavor(self, mock_nova):
+    def test_create_flavor(self):
+        self.useFixture(keystoneauth_betamax.BetamaxFixture(
+            cassette_name='test_create_flavor',
+            cassette_library_dir=self.fixtures_directory,
+            record=self.record_fixtures,
+            serializer=serializer.YamlJsonSerializer))
+
+        old_flavors = self.op_cloud.list_flavors()
         self.op_cloud.create_flavor(
             'vanilla', 12345, 4, 100
         )
-        mock_nova.flavors.create.assert_called_once_with(
-            name='vanilla', ram=12345, vcpus=4, disk=100,
-            flavorid='auto', ephemeral=0, swap=0, rxtx_factor=1.0,
-            is_public=True
-        )
 
+        # test that we have a new flavor added
+        new_flavors = self.op_cloud.list_flavors()
+        self.assertEquals(len(new_flavors) - len(old_flavors), 1)
+
+        # test that new flavor is created correctly
+        found = False
+        for flavor in new_flavors:
+            if flavor['name'] == 'vanilla':
+                found = True
+                break
+        self.assertTrue(found)
+        needed_keys = {'name', 'ram', 'vcpus', 'id', 'is_public', 'disk'}
+        if found:
+            # check flavor content
+            self.assertTrue(needed_keys.issubset(flavor.keys()))
+
+        # delete created flavor
+        self.op_cloud.delete_flavor('vanilla')
+
+    @mock.patch.object(shade.OpenStackCloud, '_compute_client')
     @mock.patch.object(shade.OpenStackCloud, 'nova_client')
-    def test_delete_flavor(self, mock_nova):
+    def test_delete_flavor(self, mock_nova, mock_compute):
+        mock_response = mock.Mock()
+        mock_response.json.return_value = dict(extra_specs=[])
+        mock_compute.get.return_value = mock_response
         mock_nova.flavors.list.return_value = [
-            fakes.FakeFlavor('123', 'lemon')
+            fakes.FakeFlavor('123', 'lemon', 100)
         ]
         self.assertTrue(self.op_cloud.delete_flavor('lemon'))
         mock_nova.flavors.delete.assert_called_once_with(flavor='123')
@@ -54,7 +79,7 @@ class TestFlavors(base.TestCase):
     @mock.patch.object(shade.OpenStackCloud, 'nova_client')
     def test_delete_flavor_exception(self, mock_nova):
         mock_nova.flavors.list.return_value = [
-            fakes.FakeFlavor('123', 'lemon')
+            fakes.FakeFlavor('123', 'lemon', 100)
         ]
         mock_nova.flavors.delete.side_effect = Exception()
         self.assertRaises(shade.OpenStackCloudException,
@@ -65,23 +90,25 @@ class TestFlavors(base.TestCase):
         self.op_cloud.list_flavors()
         mock_nova.flavors.list.assert_called_once_with(is_public=None)
 
-    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
-    def test_set_flavor_specs(self, mock_nova):
-        flavor = mock.Mock(id=1, name='orange')
-        mock_nova.flavors.get.return_value = flavor
+    @mock.patch.object(shade.OpenStackCloud, '_compute_client')
+    def test_set_flavor_specs(self, mock_compute):
         extra_specs = dict(key1='value1')
         self.op_cloud.set_flavor_specs(1, extra_specs)
-        mock_nova.flavors.get.assert_called_once_with(flavor=1)
-        flavor.set_keys.assert_called_once_with(extra_specs)
+        mock_compute.post.assert_called_once_with(
+            '/flavors/{id}/os-extra_specs'.format(id=1),
+            json=dict(extra_specs=extra_specs))
 
-    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
-    def test_unset_flavor_specs(self, mock_nova):
-        flavor = mock.Mock(id=1, name='orange')
-        mock_nova.flavors.get.return_value = flavor
+    @mock.patch.object(shade.OpenStackCloud, '_compute_client')
+    def test_unset_flavor_specs(self, mock_compute):
         keys = ['key1', 'key2']
         self.op_cloud.unset_flavor_specs(1, keys)
-        mock_nova.flavors.get.assert_called_once_with(flavor=1)
-        flavor.unset_keys.assert_called_once_with(keys)
+        api_spec = '/flavors/{id}/os-extra_specs/{key}'
+        self.assertEqual(
+            mock_compute.delete.call_args_list[0],
+            mock.call(api_spec.format(id=1, key='key1')))
+        self.assertEqual(
+            mock_compute.delete.call_args_list[1],
+            mock.call(api_spec.format(id=1, key='key2')))
 
     @mock.patch.object(shade.OpenStackCloud, 'nova_client')
     def test_add_flavor_access(self, mock_nova):
